// VFD Clock V4 using IV-27M VFD tube
// Last Updated: 06-01-20
// Displays Time, Date, Temperature, Humidity, Pressure, Alarm
// function added.
/*
Let 'day_number' be the day of the month.
Let "month_number' be the month, counting March as 1 and February as 12. 
Let 'year2_number' be the last two digits of the year.
Let 'year1_number' be the first two digits of the year.
jan = 1, feb = 2, mar = 3, apl = 4, may = 5, jun = 6,
jul = 7, aug = 8, sept = 9, oct = 10, nov = 11, dec = 12
Sat = 0, Sun = 1, Mon = 2, Tue = 3, Wed = 4, Thur = 5, Fri = 6
*/ 

#include <SPI.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x27 for a 16 chars and 2 line display 0x27,20,4

#include <Wire.h>
#include <SparkFunBME280.h>

#define SEALEVELPRESSURE_HPA (1013.25)

BME280 bme; //Uses I2C address 0x76 (jumper closed)

#include <DS3231.h>
DS3231  rtc(A4, A5); // SDA, SCL

/* two dimentional array for holdinPing MAX6921 chip dataPinbyte values
   Grid 1  Grid 2  Grid 3  Grid 4  Grid 5  Grid 6  Grid 7  Grid 8  Grid 9  Grid 10 Grid 11 Grid 12 
   OUT11    OUT10    OUT9    OUT8    OUT7    OUT6    OUT5    OUT4    OUT3    OUT2    OUT1   OUT0 
PIN  11       12       17     18      19      20      21      22      23      24      25      26
      
   seg. A  seg. B  seg. C  seg. D  seg. E  seg. F  seg. G  seg. DP
   OUT12   OUT13   OUT14   OUT15   OUT16   OUT17   OUT18   OUT19
PIN  10       9       8       7       6       5       4       3

*/
int rows = 0;
int columns = 0;
int clockPin = 43;
int loadPin = 45;
int dinPin = 47;
int dataPinRegister[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int offset;

//A F B G E C D,dp
int segmentArray[45][8]={
{ 1,1,1,0,1,1,1,0,  }, //0          
{ 0,0,1,0,0,1,0,0,  }, //1        
{ 1,0,1,1,1,0,1,0,  }, //2      
{ 1,0,1,1,0,1,1,0,  }, //3         
{ 0,1,1,1,0,1,0,0,  }, //4      
{ 1,1,0,1,0,1,1,0,  }, //5       
{ 1,1,0,1,1,1,1,0,  }, //6        
{ 1,0,1,0,0,1,0,0,  }, //7          
{ 1,1,1,1,1,1,1,0,  }, //8          
{ 1,1,1,1,0,1,0,0,  }, //9          
{ 1,1,1,0,1,1,1,1,  }, //0.          
{ 0,0,1,0,0,1,0,1,  }, //1.        
{ 1,0,1,1,1,0,1,1,  }, //2.      
{ 1,0,1,1,0,1,1,1,  }, //3.         
{ 0,1,1,1,0,1,0,1,  }, //4.      
{ 1,1,0,1,0,1,1,1,  }, //5.       
{ 1,1,0,1,1,1,1,1,  }, //6.        
{ 1,0,1,0,0,1,0,1,  }, //7.          
{ 1,1,1,1,1,1,1,1,  }, //8.          
{ 1,1,1,1,0,1,0,1,  }, //9.  
{ 0,0,0,1,0,0,0,0,  }, //-          
{ 1,1,0,0,1,0,1,0,  }, //C          
{ 1,1,0,1,1,0,0,0,  }, //F           
{ 1,1,1,1,1,0,0,0,  }, //P                            
{ 0,0,0,0,0,0,0,1,  }, //.
{ 1,1,1,1,0,0,0,0,  }, //Upper o           
{ 0,0,1,1,1,0,0,0,  }, // Slash
{ 0,0,0,1,1,1,1,0,  }, //Lower o
{ 0,0,0,0,0,0,0,0,  }, // Blank
{ 0,0,1,1,1,1,1,0,  }, // D 29
{ 1,1,1,1,1,1,0,0,  }, // A 30
{ 0,1,1,1,0,1,1,0,  }, // Y 31
{ 1,1,1,0,1,1,0,0,  }, // M first part 32
{ 1,0,1,0,0,1,0,0,  }, // X second part 33
{ 0,0,0,1,1,1,0,0,  }, // N 34
{ 0,1,0,1,1,0,1,0,  }, // T 35
{ 0,1,1,0,1,1,1,0,  }, // U 36
{ 1,1,0,1,1,0,1,0,  }, // E 37
{ 1,1,0,1,0,1,1,0,  }, // S 38
{ 0,1,1,0,1,1,1,0,  }, // W first part 39
{ 0,0,1,0,0,1,1,0,  }, // V second part 40
{ 0,1,1,1,1,1,0,0,  }, // H 41
{ 0,0,0,1,1,0,0,0,  }, // R 42
{ 1,1,1,0,1,1,1,0,  }, // Z 43 special character for O
{ 0,1,0,0,1,0,0,0,  }}; // I 44

char dayArray[7][10]={   
{ 'S','A','T','U','R','D','A','Y',' ',' '  }, //Saturday  
{ 'S','U','N','D','A','Y',' ',' ',' ',' '  }, //Sunday          
{ 'M','X','Z','N','D','A','Y',' ',' ',' '  }, //Monday M character into two parts using special X character    
{ 'T','U','E','S','D','A','Y',' ',' ',' '  }, //Tuesday       
{ 'W','V','E','D','N','E','S','D','A','Y'  }, //Wednesday W character into two parts using special V character   
{ 'T','H','U','R','S','D','A','Y',' ',' '  }, //Thursday   
{ 'F','R','I','D','A','Y',' ',' ',' ',' '  }}; //Friday     

#include <Keypad.h>

char key_matrix[4][4] = {
  {'1','4','7','*'},
  {'2','5','8','0'},
  {'3','6','9','#'},
  {'A','B','C','D'}
};
// Set up the pins used for scanning.
// Don't use 0 or 1 (serial port)
uint8_t row_pins[4] = {5, 4, 3, 2};
uint8_t col_pins[4] = {9, 8, 7, 6};
char key_pressed;
int field_value;

// Instantiate a Keypad object.
Keypad mykeypad = Keypad( makeKeymap(key_matrix), row_pins, col_pins, 4, 4); 

//Specify digital pin on the Arduino that the positive lead of piezo buzzer is attached.
int piezoPin = 10;
int days = 11;
int months = 3;
int years = 19;
int new_days = 11;
int new_months = 3;
int new_years = 19;
int hours = 16;
int minutes = 11;
int seconds = 20;
int new_hours = 16;
int new_minutes = 11;
int new_seconds = 20;
int found, maxIndex;
int digits_entered = 0;
int alarm_hours = 0;
int alarm_minutes = 0;
int alarm_hours_check;
int alarm_minutes_check;
int j,x,y,k,m,n,i,t,index,a;
int day_number, month_number, year1_number, year2_number,day_of_the_week, working_day;
char testChar;

String new_value_1, new_value_2;
String dayString;
String monthString;
String yearString;
String day1,day2;
String month1,month2;
String year1,year2,year3, year4;
String hourString;
String minuteString;
String secondString;
String hour1;
String minute1;
String second1;
String hour2;
String minute2;
String second2;
String year_string,date_string;
String timeString, dateString;
String characterArray[11]; // Character Array
String humidityString;
String humidityString1;
String humidityString2;
String pressureString;
String pressureString1;
String pressureString2;
String temperatureCString;
String temperatureCString1;
String temperatureCString2;
String temperatureFString;
String temperatureFString1;
String temperatureFString2;
float pressure,temperatureC, temperatureF, humidity;

void setup()
{
  Serial.begin(9600);
  // initialize outputs for MAX6921 chip
  pinMode(dinPin, OUTPUT);
  pinMode(loadPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  digitalWrite(clockPin, LOW);
  digitalWrite(loadPin, LOW);
  digitalWrite(dinPin, LOW);
  
  // initialize the lcd 
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print(" VFD Clock V4.0 "); 
  Serial.begin(9600);
  rtc.begin(); // Initialize the rtc object
//  rtc.setTime(9, 35, 0);     // Set the time only for first test run
//  rtc.setDate(24, 5, 2019);   // Set the date  
  Wire.begin();
  bme.setI2CAddress(0x76); //Connect to a second sensor 0x76
  if(bme.beginI2C() == false) 
  {
    Serial.println("Sensor B connect failed"); 
  }
  
  lcd.setCursor(0,0);
  lcd.print(" VFD Clock V4.0 ");
}

void loop()
{
    // Display time for 30 seconds
    display_time();
    lcd_display(); 
    for (t = 1; t <= 2000; t++) // equates to a display time of 30 seconds 1400
    {    
          check_user_input();
          display_time();  // Display time on VFD screen
    }
    alarm_hours_check = hourString.toInt();
    alarm_minutes_check = minuteString.toInt();
    // check for alarm time and if alarm has been set
    if (alarm_hours != 0 and alarm_hours != 0)
       {
          if (alarm_hours_check == alarm_hours and alarm_minutes_check == alarm_minutes)
             {
                alarm();
             }
       }   
       
    // Display date    
    dayString = getValue(rtc.getDateStr(), '.', 0);
    display_date();  // Display date on VFD screen
    lcd_display();
    for (t = 1; t <= 200; t++) // equates to a display time of 5 seconds
    {         
          check_user_input();
          display_date();  // Display date on VFD screen
    } 

    // Display name of day of the week
    display_day_of_the_week();
    lcd_display();
    for (t = 1; t <= 200; t++) // equates to a display time of 5 seconds
    {         
          check_user_input();
          display_day_of_the_week();  // Display day of the week on VFD screen
    } 

    // Display temperature in Celisus
    temperatureC = bme.readTempC();   
    display_temperatureC();  // Display temperatureC on VFD screen  
    lcd_display(); 
    for (t = 1; t <= 200; t++) 
    {      
          check_user_input();
          display_temperatureC();  // Display temperatureC on VFD screen     
    }

    // Display temperature in Farenheit
    temperatureF = bme.readTempF();   
    display_temperatureF();  // Display temperatureF on VFD screen 
    lcd_display();
    for (t = 1; t <= 200; t++) 
    {        
          check_user_input();      
          display_temperatureF();  // Display temperatureF on VFD screen 
    }

    // Display pressure in mB
    pressure = bme.readFloatPressure()/100.0F;  
    display_pressure();  // Display pressure on VFD screen  
    lcd_display();    
    for (t = 1; t <= 200; t++)
    {
          check_user_input(); 
          display_pressure();  // Display pressure on VFD screen         
    }

    // Display humidity
    humidity = bme.readFloatHumidity();
    display_humidity();  // Display humidity on VFD screen       
    lcd_display();
    for (t = 1; t <= 200; t++) 
    {       
          check_user_input(); 
          display_humidity();  // Display humidity on VFD screen   
    }        
}

void alarm()
{
  for (i = 0; i <= 5; i++)
  {  
    /*Tone needs 2 arguments, but can take three
      1) Pin#
      2) Frequency - this is in hertz (cycles per second) which determines the pitch of the noise made
      3) Duration - how long teh tone plays
    */
    tone(piezoPin, 1000, 500);
    delay(500);
    tone(piezoPin, 750, 500);
    delay(500);
  }
}

void check_user_input()
{
  if ((key_pressed = mykeypad.getKey()) != false)
  {
    if (key_pressed == '#')
    {
      adjustValues();
    }  
  }
}

void lcd_display() // Display on the LCD exactly what is stored in the character array.
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(" VFD Clock V4.0 ");
  lcd.setCursor(3,1);
  for (y = 0; y <=9; y++)
  {
     if (characterArray[y] == "O")
        lcd.print((char) 223);
     else 
        lcd.print(characterArray[y]);
  }
  lcd.println("   ");
}

void display_time()
{
  hourString = getValue(rtc.getTimeStr(), ':', 0);
  hour1 = hourString.substring(0,1);
  hour2 = hourString.substring(1,2);  
  minuteString = getValue(rtc.getTimeStr(), ':', 1);
  minute1 = minuteString.substring(0,1);
  minute2 = minuteString.substring(1,2);  
  secondString = getValue(rtc.getTimeStr(), ':', 2);
  second1 = secondString.substring(0,1);
  second2 = secondString.substring(1,2); 
  if ( hour1 == "0" )
  {
    hour1 = " ";
  }
  characterArray[0] = hour1;
  characterArray[1] = hour2; 
  characterArray[2] = ".";
  characterArray[3] = minute1; 
  characterArray[4] = minute2; 
  characterArray[5] = ".";   
  characterArray[6] = second1; 
  characterArray[7] = second2;   
  characterArray[8] = " ";
  characterArray[9] = " ";  
  process_display();       
}

void display_day_of_the_week()
{
// Sat = 0, Sun = 1, Mon = 2, Tue = 3, Wed = 4, Thur = 5, Fri = 6
  day_number = dayString.substring(0,2).toInt(); 
  month_number = monthString.substring(0,2).toInt(); 
  year1_number = yearString.substring(0,2).toInt();
  year2_number = yearString.substring(2,4).toInt(); 

  day_of_the_week = year2_number;
  day_of_the_week = day_of_the_week/4;
  day_of_the_week = day_of_the_week + day_number;
  // Month Key correction, April and July have not correction
  if(month_number == 1 || month_number == 10)
  {
    day_of_the_week = day_of_the_week + 1;
  }
  if(month_number == 2 || month_number == 3 || month_number == 11)
  {
    day_of_the_week = day_of_the_week + 4;
  }
  if(month_number == 5)
  {
    day_of_the_week = day_of_the_week + 2;
  }  
  if(month_number == 6)
  {
    day_of_the_week = day_of_the_week + 5;
  }  
  if(month_number == 8)
  {
    day_of_the_week = day_of_the_week + 3;
  }  
  if(month_number == 9 || month_number == 12)
  {
    day_of_the_week = day_of_the_week + 6;
  }  
  // Test for leap year, and subtract 1 if it is
  if(year2_number%4 == 0)
  {
    if (month_number == 1 || month_number == 2)
    {
        day_of_the_week = day_of_the_week - 1;
    }
  }
  // end
  // Fixed 2000 centry correction
  day_of_the_week = day_of_the_week + 6;
  // end
  day_of_the_week = day_of_the_week + year2_number;
  day_of_the_week = day_of_the_week%7;
  
  for (a = 0; a <=9; a++)
  {
     characterArray[a] = dayArray[day_of_the_week][a];                
  }
  process_display();
} 
   
void display_date()
{
  day1 = dayString.substring(0,1);
  day2 = dayString.substring(1,2);  
  monthString = getValue(rtc.getDateStr(), '.', 1);
  month1 = monthString.substring(0,1);
  month2 = monthString.substring(1,2);  
  yearString = getValue(rtc.getDateStr(), '.', 2);
  year1 = yearString.substring(0,1);
  year2 = yearString.substring(1,2); 
  year3 = yearString.substring(2,3);
  year4 = yearString.substring(3,4); 
  characterArray[0] = day1;  
  characterArray[1] = day2;
  characterArray[2] = "-";
  characterArray[3] = month1; 
  characterArray[4] = month2;
  characterArray[5] = "-"; 
  characterArray[6] = year3;
  characterArray[7] = year4;
  characterArray[8] = " ";  
  characterArray[9] = " ";  
  process_display();
}

void display_temperatureC()
{ 
  temperatureCString = String(temperatureC);  
  temperatureCString1 = getValue(temperatureCString, '.', 0); // 99.
  temperatureCString2 = getValue(temperatureCString, '.', 1); // .99
  characterArray[0] = " ";   
  characterArray[1] = temperatureCString1.substring(0,1);
  characterArray[2] = temperatureCString1.substring(1,2);  
  characterArray[3] = ".";
  characterArray[4] = temperatureCString2.substring(0,1); 
  characterArray[5] = temperatureCString2.substring(1,2);
  characterArray[6] = " ";
  characterArray[7] = "O";
  characterArray[8] = "C";
  characterArray[9] = " ";      
  process_display();
}

void display_temperatureF()
{ 
  temperatureFString = String(temperatureF);  
  temperatureFString1 = getValue(temperatureFString, '.', 0); // 99.
  temperatureFString2 = getValue(temperatureFString, '.', 1); // .99
  characterArray[0] = " ";   
  characterArray[1] = temperatureFString1.substring(0,1);
  characterArray[2] = temperatureFString1.substring(1,2);  
  characterArray[3] = ".";
  characterArray[4] = temperatureFString2.substring(0,1); 
  characterArray[5] = temperatureFString2.substring(1,2);
  characterArray[6] = " ";
  characterArray[7] = "O";
  characterArray[8] = "F";
  characterArray[9] = " ";  
  process_display();  
}

void display_pressure()
{
  pressureString = String(pressure);
  pressureString1 = getValue(pressureString, '.', 0); // 9999.
  pressureString2 = getValue(pressureString, '.', 1); // .99
  if (pressureString1.substring(0,1) == "9")  // correctly display pressure values less than 1000 mB
    {
      characterArray[0] = " ";
      characterArray[1] = pressureString1.substring(0,1); 
      characterArray[2] = pressureString1.substring(1,2);
      characterArray[3] = pressureString1.substring(2,3); 
    }
  else
    {  
      characterArray[0] = pressureString1.substring(0,1); 
      characterArray[1] = pressureString1.substring(1,2);
      characterArray[2] = pressureString1.substring(2,3); 
      characterArray[3] = pressureString1.substring(3,4); 
    }   
  characterArray[4] = ".";  
  characterArray[5] = pressureString2.substring(0,1);  
  characterArray[6] = pressureString2.substring(1,2);
  characterArray[7] = " ";  
  characterArray[8] = "P";  
  characterArray[9] = " ";     
  process_display(); 
}

void display_humidity()
{
  humidityString = String(humidity);
  humidityString1 = getValue(humidityString, '.', 0); // 99.
  humidityString2 = getValue(humidityString, '.', 1); // .99
  characterArray[0] = humidityString2.substring(0,1); 
  characterArray[1] = humidityString2.substring(1,2); 
  characterArray[2] = ".";
  characterArray[3] = humidityString1.substring(0,1);
  characterArray[4] = humidityString1.substring(1,2);
  characterArray[5] = " ";
  characterArray[6] = "o";
  characterArray[7] = "/";  
  characterArray[8] = "O";  
  characterArray[9] = " ";
  process_display(); 
}

void adjustValues()
{
      // get current time and date, populate adjustment fields.
      hourString = getValue(rtc.getTimeStr(), ':', 0);
      minuteString = getValue(rtc.getTimeStr(), ':', 1);
      secondString = getValue(rtc.getTimeStr(), ':', 2);
      hour1 = hourString.substring(0,2);
      minute1 = minuteString.substring(0,2);  
      second1 = secondString.substring(0,2);
              
      dayString = getValue(rtc.getDateStr(), '.', 0);
      monthString = getValue(rtc.getDateStr(), '.', 1);
      yearString = getValue(rtc.getDateStr(), '.', 2);
      day1 = dayString.substring(0,2);
      month1 = monthString.substring(0,2);
      year1 = yearString.substring(2,4);           
      
      new_days = day1.toInt();
      new_months = month1.toInt();
      new_years = year1.toInt();
    
      new_hours = hour1.toInt();
      new_minutes = minute1.toInt();
      new_seconds = second1.toInt();
    
         lcd.clear();
         lcd.setCursor(0,0);            
         lcd.print("Clock Adjustment");
         delay(2000);
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("   Days(DD): ");
         lcd.print(new_days);
         adjust_field(1);
         delay(1000);
         
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print(" Months(MM): ");
         lcd.print(new_months);
         adjust_field(2);
         delay(1000); 
                 
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("  Years(YY): ");
         lcd.print(new_years);
         adjust_field(3);
         delay(1000);

         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("  Hours(HH): ");
         lcd.print(new_hours);
         adjust_field(4);
         delay(1000);
         
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("Minutes(MM): ");
         lcd.print(new_minutes);
         adjust_field(5);
         delay(1000);   
               
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("Seconds(SS): ");
         lcd.print(new_seconds);
         adjust_field(6);
         delay(1000);   

         alarm_hours = 0;
         alarm_minutes = 0;

         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("  Alarm(HH): ");
         lcd.print(alarm_hours);
         adjust_field(7);
         delay(1000);
         
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print("  Alarm(MM): ");
         lcd.print(alarm_minutes);
         adjust_field(8);
         delay(1000);   
         
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print(" New Values Set");
         new_years = 2000 + new_years;         
         rtc.setTime(new_hours, new_minutes, new_seconds);     // Set the time
         rtc.setDate(new_days, new_months, new_years);         // Set the date  
}

void adjust_field(int field)
{
  field_value = field;
  digits_entered = 13;
  key_pressed = mykeypad.getKey();
    while (key_pressed != '#')
    {
      if (key_pressed == 'A' or key_pressed == 'B' or key_pressed == 'C'
          or key_pressed == 'D' or key_pressed == '*')
      {
            display_Error(1);     
      }
      if (key_pressed == '0' or key_pressed == '1' or key_pressed == '2' or
          key_pressed == '3' or key_pressed == '4' or key_pressed == '5' or 
          key_pressed == '6' or key_pressed == '7' or key_pressed == '8' or
          key_pressed == '9')
          {
            if (digits_entered == 13 )
            {
                 lcd.setCursor(digits_entered,0);
                 digits_entered = digits_entered + 1;
                 lcd.print(key_pressed);
            } else
              { 
              if (digits_entered == 14 )
                  {
                       lcd.setCursor(digits_entered,0);
                       digits_entered = digits_entered + 1;        
                       lcd.print(key_pressed);
                  } else
                      {
                        display_Error(2);
                        digits_entered = 13;
                      }
              }                       
            switch (field_value)
            {
              case 1:
                if (digits_entered == 14 )
                {
                  new_days = (int)key_pressed - 48;
                } else 
                {
                  new_days = (new_days * 10) + (int)key_pressed - 48;    
                }         
              break; 
              case 2:
                if (digits_entered == 14 )
                {
                  new_months = (int)key_pressed - 48;
                } else 
                {
                  new_months = (new_months * 10) + (int)key_pressed - 48;    
                }
              break; 
              case 3:
                if (digits_entered == 14 )
                {
                  new_years = (int)key_pressed - 48;
                } else 
                {
                  new_years = (new_years * 10) + (int)key_pressed - 48;    
                }        
              break;          
              case 4:
                if (digits_entered == 14 )
                {
                  new_hours = (int)key_pressed - 48;
                } else 
                {
                  new_hours = (new_hours * 10) + (int)key_pressed - 48;    
                }
              break;
              case 5:
                if (digits_entered == 14 )
                {
                  new_minutes = (int)key_pressed - 48;
                } else 
                {
                  new_minutes = (new_minutes * 10) + (int)key_pressed - 48;    
                }
              break;
              case 6:
                if (digits_entered == 14 )
                {
                  new_seconds = (int)key_pressed - 48;
                } else 
                {
                  new_seconds = (new_seconds * 10) + (int)key_pressed - 48;    
                }               
              break;  
              case 7:
                if (digits_entered == 14 )
                {
                  alarm_hours = (int)key_pressed - 48;
                } else 
                {
                  alarm_hours = (alarm_hours * 10) + (int)key_pressed - 48;    
                } 
              break;
              case 8:
                if (digits_entered == 14 )
                {
                  alarm_minutes = (int)key_pressed - 48;
                } else 
                {
                  alarm_minutes = (alarm_minutes * 10) + (int)key_pressed - 48;    
                }               
              break;          
            }
          }  
            key_pressed = mykeypad.getKey();
            // Wait for user to press a key on the keypad
            while (key_pressed == NO_KEY)
            {
                key_pressed = mykeypad.getKey(); 
                delay(100);
            }
    }
}

void display_Error(int type)
{
   lcd.setCursor(0,1);
   if (type == 1 )
   {
    lcd.print("  Invalid Entry ");   
   } else 
   {
    lcd.print("Too many digits ");    
   }
   delay (2000);
   lcd.setCursor(0,1);
   lcd.print("                ");      
   lcd.setCursor(0,0);
  
   if (field_value == 1)
   {
         lcd.print("   Days(DD): ");    
         new_days = day1.toInt();
         lcd.print(new_days);         
   }
   if (field_value == 2)
   {
         lcd.setCursor(0,0);
         lcd.print("                "); 
         lcd.setCursor(0,0);   
         lcd.print(" Months(MM): ");    
         new_months = month1.toInt();
         lcd.print(new_months);         
   }
   if (field_value == 3)
   {
         lcd.print("  Years(YY): ");    
         new_years = year1.toInt();
         lcd.print(new_years);         
   }
   if (field_value == 4)
   {
         lcd.print("  Hours(HH): ");
         new_hours = hour1.toInt();
         lcd.print(new_hours);         
   }      
   if (field_value == 5)
   {
         lcd.print("Minutes(MM): ");
         new_minutes = minute1.toInt();
         lcd.print(new_minutes);         
   }      
   if (field_value == 6)
   {
         lcd.print("Seconds(SS): ");
         new_seconds = second1.toInt();
         lcd.print(new_seconds);         
   }      
   if (field_value == 7)
   {
         lcd.print("  Alarm(HH): ");
         lcd.print(alarm_hours);         
   }      
   if (field_value == 8)
   {
         lcd.print("  Alarm(MM): ");
         lcd.print(alarm_minutes);         
   } 
}

String getValue(String dataPin, char separator, int index)
{
    found = 0;
    int strIndex[] = { 0, -1 };
    maxIndex = dataPin.length() - 1;
    for (i = 0; i <= maxIndex && found <= index; i++) 
    {
        if (dataPin.charAt(i) == separator || i == maxIndex) 
        {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? dataPin.substring(strIndex[0], strIndex[1]) : "";
}

void write_loadPin()
{
    digitalWrite(loadPin, HIGH);
    digitalWrite(loadPin, LOW);  
}

void write_clockPin()
{
    digitalWrite(clockPin, HIGH);
    digitalWrite(clockPin, LOW); 
}

void write_character()
{
  if(characterArray[9-offset] == "0")
    index = 0;
  if(characterArray[9-offset] == "1")
    index = 1;
  if(characterArray[9-offset] == "2")
    index = 2;
  if(characterArray[9-offset] == "3")
    index = 3;
  if(characterArray[9-offset] == "4")
    index = 4;
  if(characterArray[9-offset] == "5")
    index = 5;
  if(characterArray[9-offset] == "6")
    index = 6;                        
  if(characterArray[9-offset] == "7")
    index = 7;
  if(characterArray[9-offset] == "8")
    index = 8;
  if(characterArray[9-offset] == "9")
    index = 9; 
  if(characterArray[9-offset] == "0.")
    index = 10;
  if(characterArray[9-offset] == "1.")
    index = 11;
  if(characterArray[9-offset] == "2.")
    index = 12;
  if(characterArray[9-offset] == "3.")
    index = 13;
  if(characterArray[9-offset] == "4.")
    index = 14;
  if(characterArray[9-offset] == "5.")
    index = 15;
  if(characterArray[9-offset] == "6.")
    index = 16;                        
  if(characterArray[9-offset] == "7.")
    index = 17;
  if(characterArray[9-offset] == "8.")
    index = 18;
  if(characterArray[9-offset] == "9.")
    index = 19;     
  if(characterArray[9-offset] == "-")
    index = 20;
  if(characterArray[9-offset] == "C")
    index = 21;
  if(characterArray[9-offset] == "F")
    index = 22;
  if(characterArray[9-offset] == "P")
    index = 23;
  if(characterArray[9-offset] == ".")
    index = 24;
  if(characterArray[9-offset] == "O")
    index = 25;                        
  if(characterArray[9-offset] == "/")
    index = 26;
  if(characterArray[9-offset] == "o")
    index = 27;
  if(characterArray[9-offset] == " ")
    index = 28;
  if(characterArray[9-offset] == "D")
    index = 29;
  if(characterArray[9-offset] == "A")
    index = 30;
  if(characterArray[9-offset] == "Y")
    index = 31;
  if(characterArray[9-offset] == "M") // first part of M character
    index = 32;
  if(characterArray[9-offset] == "X") // second part of M character
    index = 33;
  if(characterArray[9-offset] == "N")
    index = 34;                        
  if(characterArray[9-offset] == "T")
    index = 35;
  if(characterArray[9-offset] == "U")
    index = 36;
  if(characterArray[9-offset] == "E")
    index = 37; 
  if(characterArray[9-offset] == "S")
    index = 38;
  if(characterArray[9-offset] == "W") // first part of W character
    index = 39;
  if(characterArray[9-offset] == "V") // second part of W character
    index = 40;
  if(characterArray[9-offset] == "H")
    index = 41;
  if(characterArray[9-offset] == "R")
    index = 42;
  if(characterArray[9-offset] == "Z")
    index = 43;
  if(characterArray[9-offset] == "I")
    index = 44;                    
}

void process_display()
{
    for (offset = 9; offset >= 0; offset--)
    {
      write_character();
      write_clockPin();      
      for (i = 19; i >= 0; i--)
      {
        if (i >= 12)
        {
          dataPinRegister[i] = segmentArray[index][i-12]; 
        }
        digitalWrite(dinPin, dataPinRegister[i]);         
        write_clockPin();
      } 
      write_loadPin();
      dataPinRegister[offset] = 0;
      if (offset != 0)
          dataPinRegister[offset-1] = 1; 
      else
          dataPinRegister[9] = 1;
      delay(1);   
    }
}
